@@ -3,6 +3,7 @@ import psycopg2.sql as sql
 
 from Connector import Connector
 from error_handlers import log_error, retry_log_error
+from qrlogging import logger
 
 
 class PostgresConnector(Connector):
@@ -23,10 +24,12 @@ class PostgresConnector(Connector):
     @log_error
     def exec(self, request: str, identifiers: list = None, literals: list = None, result='all'):
         super().exec(request, identifiers, literals, result)
+        request = sql.SQL(request)
         if identifiers:
             identifiers = [sql.Identifier(x) for x in identifiers]
-            request = sql.SQL(request).format(*identifiers)
+            request = request.format(*identifiers)
 
+        logger.warning('POSTGRES EXECUTE: %s with literals %s', request.as_string(self.cursor), literals)
         self.cursor.execute(request, literals)
         return self.extract_result(result)
 
