@@ -1,6 +1,7 @@
 # todo load from config data format -> refactor
 from error_handlers import log_error
-
+from qrlogging import logger
+# todo error: tuple index out of range here on line 22
 
 class DataFormatter:
     def __init__(self):
@@ -9,7 +10,9 @@ class DataFormatter:
     # result type -> 'all', 'one'
     @log_error
     def format_data(self, data, used_fields, result_type):
-        if result_type is None:
+        logger.warning("Formatting:\n\tdata: %s\n\tused_fields: %s\n\tresult: %s", data,
+                    used_fields, result_type)
+        if result_type is None or data is None:
             return None
 
         if self.format == 'list':
