@@ -17,8 +17,9 @@ def main():
     data = books.select('count(*)').group_by(books.original_publication_year).all()
     print(data[:10])
 
-    data = DB.select(books, books.id).where('id < 10').order_by(books.id, desc=True).one()
-    print(data)
+    data = DB.select(books, books.id).where('id < 10').order_by(books.id, desc=True).\
+        limit(3).offset(2).all()
+    print('limit & offset', data)
 
     data = books.select(authors.id, books.id)\
         .join(books_authors, 'books_authors.book_id = books.id')\
@@ -43,6 +44,9 @@ def main():
     data = query.all()
     print(data)
 
+    data = DB.exec('select * from get_book_authors(1) as f(id int, name varchar)').config_fields('id', 'name').all()
+    print(data)
+
 
 if __name__ == '__main__':
     main()
\ No newline at end of file
