@@ -9,8 +9,9 @@ from Connector import Connector
 
 # accurate - db.operators needed with 'db.' to avoid different namespaces-> isinstance will fail
 
-# todo unsafe warnings - deal with security on raw strings
+# todo unsafe warnings - deal with security on raw strings and others
 
+# todo where not only by field_name (may be dubious)
 
 def request_operators_order(op):
     if op == 'join': return 0
@@ -21,6 +22,8 @@ def request_operators_order(op):
     elif op == 'group_by': return 2
     elif op == 'having': return 3
     elif op == 'order_by': return 4
+    elif op == 'limit': return 5
+    elif op == 'offset': return 6
 
 
 def get_field(field_name, tables) -> QRField:
@@ -55,7 +58,7 @@ def parse_request_args(tables, *args, disable_full_name=False, **kwargs):
         conditions.append(condition)
 
     if args:
-        logger.warning("UNSAFE: executing raw select from table %s:  'where %s'", tables[0], args)
+        #logger.warning("UNSAFE: executing raw select from table %s:  'where %s'", tables[0], args)
         for arg in args:
             conditions.append(arg)
 
@@ -64,7 +67,7 @@ def parse_request_args(tables, *args, disable_full_name=False, **kwargs):
 
 @log_class(log_error_default_self)
 class QRequest:
-    def __init__(self, connector: Connector, table: QRTable, request: str = '',
+    def __init__(self, connector: Connector, table: QRTable = None, request: str = '',
                  identifiers=None, literals=None, auto_commit=False):
         self.connector = connector
         self.request = request
@@ -75,7 +78,7 @@ class QRequest:
 
         self.identifiers = identifiers
         self.literals = literals
-        self.tables = [table]
+        self.tables = [table] if table else []
         self.conditions = dict()
         self.auto_commit = auto_commit
         self.cur_order = -1
@@ -86,11 +89,27 @@ class QRequest:
         cond_ops.sort(key=lambda x: request_operators_order(x))
         for ext in [self.conditions[i] for i in cond_ops]:
             self.request += ' ' + ext
-        self.connector.exec(self.request, self.identifiers, self.literals, result=result)
+
+        logger.warning('EXECUTING REQUEST WITH:\n\trequest=%s\n\tused_fields=%s\n\tidents=%s\n\tliterals=%s\n\t',
+                       self.request, self.used_fields, self.identifiers, self.literals)
+        data = self.connector.exec(self.request, self.identifiers, self.literals, result=result)
 
         if self.auto_commit:
             self.connector.commit()
-        return True
+
+        return True if (result is None) else \
+            defaultDataFormatter.format_data(data, self.used_fields, result)
+
+    def config_fields(self, *args):
+        # todo add support for iterable params
+        self.used_fields = list(args)
+        return self
+
+    def all(self):
+        return self.exec('all')
+
+    def one(self):
+        return self.exec('one')
 
 
 @log_class(log_error_default_self)
@@ -132,14 +151,8 @@ class QRSelect(QRWhere):
             self.used_fields = list(used_fields)
         else:
             self.used_fields = list(table.meta['fields'].keys())
+            # todo unsafe - order may not be the same
 
-    def exec(self, result=None):
-        cond_ops = list(self.conditions.keys())
-        cond_ops.sort(key=lambda x: request_operators_order(x))
-        for ext in [self.conditions[i] for i in cond_ops]:
-            self.request += ' ' + ext
-        data = self.connector.exec(self.request, self.identifiers, self.literals, result=result)
-        return defaultDataFormatter.format_data(data, self.used_fields, result)
 
     def group_by(self, *args):
         if request_operators_order('group_by') < self.cur_order:
@@ -193,6 +206,32 @@ class QRSelect(QRWhere):
 
         return self
 
+    def limit(self, n):
+        if request_operators_order('limit') < self.cur_order:
+            raise Exception('select: wrong operators sequence')
+        self.cur_order = max(self.cur_order, request_operators_order('limit')) + 1
+
+        if not isinstance(n, int):
+            raise Exception('select: integer expected as a limit param')
+
+        self.literals.append(n)
+        self.conditions['limit'] = ' limit %s '
+
+        return self
+
+    def offset(self, n):
+        if request_operators_order('offset') < self.cur_order:
+            raise Exception('select: wrong operators sequence')
+        self.cur_order = max(self.cur_order, request_operators_order('offset')) + 1
+
+        if not isinstance(n, int):
+            raise Exception('select: integer expected as a offset param')
+
+        self.literals.append(n)
+        self.conditions['offset'] = ' offset %s '
+
+        return self
+
     def join(self, table: QRTable, cond):
         if request_operators_order('join') < self.cur_order:
             raise Exception('select: wrong operators sequence')
@@ -201,14 +240,14 @@ class QRSelect(QRWhere):
         self.tables.append(table)
 
         if type(cond) == str:
-            logger.warning("UNSAFE: executing raw select from table %s:  'join on %s'",
-                           self.tables[0], cond)
+            #logger.warning("UNSAFE: executing raw select from table %s:  'join on %s'",
+            #               self.tables[0], cond)
             self.identifiers.append(table.meta['table_name'])
-            join_cond = 'join {} on %s' % cond
+            join_cond = ' join {} on %s' % cond
 
         else:
-            if not isinstance(cond, op.Eq):
-                raise Exception('join: op.Eq instance expected, got %s' % type(cond))
+            # todo different imports -> no recognition if not isinstance(cond, op.Eq):
+            #    raise Exception('join: op.Eq instance expected, got %s' % type(cond))
             if not cond.duos:
                 raise Exception('join: op.Eq contains only one instance')
             if not isinstance(cond.arg1, QRField) or not isinstance(cond.arg2, QRField):
@@ -223,15 +262,9 @@ class QRSelect(QRWhere):
 
         if self.conditions.get('join') is None:
             self.conditions['join'] = ''
-        self.conditions['join'] += join_cond
+        self.conditions['join'] += ' ' + join_cond
         return self
 
-    def all(self):
-        return self.exec('all')
-
-    def one(self):
-        return self.exec('one')
-
 
 @log_class(log_error_default_self)
 class QRUpdate(QRWhere):
@@ -328,8 +361,8 @@ class QRInsert(QRequest):
                 identifiers.extend([arg.name])
                 self.used_fields.append(arg.name)
             else:
-                logger.warning('UNSAFE: executing raw returning from table %s with args: %s',
-                               self.tables[0], args)
+                #logger.warning('UNSAFE: executing raw returning from table %s with args: %s',
+                #               self.tables[0], args)
                 # todo here used fields
                 if arg == '*':
                     self.used_fields = list(self.tables[0].meta['fields'].keys())
@@ -345,10 +378,3 @@ class QRInsert(QRequest):
             else:
                 self.conditions['returning'] += ' , ' + q
             return self
-
-    def all(self):
-        return self.exec('all')
-
-    def one(self):
-        return self.exec('one')
-
